# The original PERL script was received from Bob Batzinger of UBS Singapore.
# Edited by Russ Terjung 25 Nov 96:
#
#    This PERL script is written for scripture projects where the scripture
#              text and the footnote text are in separate text files.
#              The goal is to merge these two text files into a single
#              scripture text file with the footnote references and
#              footnote text placed in the appropriate place for the
#              SFConverter, an SIL program available from JAARS Inc.
#
#    PURPOSE: 1) to support merging separate footnote and scripture
#                files into a single scripture file ready for
#                the SFConverter and Word Scripture Template, SIL programs.
#
#             2) revised code holds footnote(s) until the complete string
#                of verse text has been processed. Thus, multiple footnotes
#                are written after the text containing the references is
#                written out to file.
#                
#             3) input verse is kept intact without adding linebreaks.
#                The user may want to edit the merged file within DOS.
#
#             4) Footnote references are edited for duplicates in file.
#
#             5) Command line file names are edited against duplication.
#
#       PERL MRGFOOT.PL <footnote input text> <scripture input text>
#                       <scripture output text> > <output file for STDOUT>
#
#       This code writes error / information messages out to STDOUT.
#       Thus, you can 're-direct' STDOUT to a file name of your choice
#       by using the '>' redirection operator at the end of the line.
#       If you do not use this operator, the messages will go to the console.
#
# MrgFoot.pl - Merge footnotes
#   inserts footnote text from another file into Bible text

if (scalar(@ARGV) != 3)
{
	print STDERR <<"usageend"; 
mrgfoot.pl - merge footnotes into Bible text  

Usage -

    perl mrgfoot.pl ftnotefile inputtextfile outputtextfile

                    OR TO WRITE MESSAGES TO FILE,

    perl mrgfoot.pl ftnotefile inputtextfile outputtextfile > messages

    Inserts footnote(s) after the text that contains 
    the footnote marker/call has been completely processed.

usageend
die '';
}
print STDERR "$ftnfile $txtfile $outfile \n";
# Check the command file names for duplicates:
if (($ARGV[0] eq $ARGV[1]) || ($ARGV[0] eq $ARGV[2]) || ($ARGV[1] eq $ARGV[2]))
{
	print STDERR <<"filerr";
mrgfoot.pl - merge footnoes into Bible text

Command line File Name Error:
	$ARGV[0] is the name of the Footnote file.
	$ARGV[1] is the name of the Text file.
	$ARGV[2] is the name of the merged Output file.
        optionally, the '>' redirection file for errors.
filerr
die '';
}

#OPEN FNOTE FILE
open(FNOTE,$ARGV[0]) || die("Not able to open FNOTE FILE:$ARGV[0]");
print(STDOUT "Processing footnotes file: $ARGV[0]\n");
#    READ EACH FNOTE INTO AN ASSOCIATE ARRAY (footnote # is key)
%ftntlst = ();          # The associative array of the Footnote File.
$statproc = 0;          # zero = outside footnote  1 = inside footnote
$errnum = 0;            # A running count of errors for EOJ reporting
while(<FNOTE>)
{
        if (/\\f([0-9]*)/) # Look for footnote of form: \f# where # is integer
	{
#               REMEMBER THE FOOTNOTE LETTER
		$mrkval = $1;
                $index = $mrkval;
                if (length($ftntlst{$index}) > 0)
                {
                        print(STDOUT "ERROR; $ARGV[0] : Footnote number $index is duplicated\n",
                                     "                  $ftntlst{$index}\n",
                                     "                  will be replaced with\n",
                                     "                  $_\n");
                        $errnum = ++$errnum;
                }
		$ftntlst{$index} = $_;
		$statproc = 1;
	}
	elsif (/\\F/)
	{
		$statproc = 0;  
	}
	elsif (/\\id/)
	{
                $statproc = 0;  
	}
	elsif ($statproc == 1)
	{
		$ftntlst{$index} .= "$_";
	}
}       
#CLOSE FNOTE FILE
close (FNOTE);

# PRINT OUT LIST OF FOOTNOTES SORTING THEM CORRECTLY
print (STDOUT "Footnotes markers found:\n", join(' ',sort(keys(%ftntlst))),"\n");
#OPEN BIBLE TXT FILE
open(TXTFILE,$ARGV[1]) || die("Not able to open TEXT FILE:$ARGV[1]");
open(OUTFILE,'>'. $ARGV[2]) || die("Not able to write the OUTFILE:$ARGV[2]"); 
print(STDOUT "Processing text file: $ARGV[1] -->$ARGV[2]\n");
#   READ IN THE TEXT LINE
$bookid = '???';
$chnum = 0;
$vsnum = 0;
%txtftlst = ();
while(<TXTFILE>)
{
        $SFMString = False;     # switch needed to determine the need for
                                # line feed BEFORE writing merged footnote
                                # string: Is footnote reference in-line
                                #         or at the end of the line?
#       NOTE THE BIBLE REF (\id, \c, \v) OF THE LINE
        if (/\\v *([0-9]+)/)    # verse format: \v # where # is integer

	{
		$vsnum = $1;
                $SFMString = True;
	}
        elsif(/\\c *([0-9]+)/)   # Chapter format: \c # where # is integer
	{
		$chnum = $1;
		$vsnum = 0;
                $SFMString = True;
 	}
        elsif(/\\id +(...)/)     # Identification: \id abc where abc is BookId
	{
		$bookid = $1;
		$chnum = 0;
		$vsnum = 0;
                $SFMString = True;
	}
#           LOOK FOR END OF FILE MARKER
	elsif (/\\z/)
	{
                $SFMString = True;
	}
#           LOOK FOR END OF FILE MARKER
	elsif (/\\s/)
	{
                $SFMString = True;
        }

#   output the Footnote from NoteBuffer
        if (/\\/ && $SFMString)  # Watch out here for cases where footnote
                                 # reference is in-line versus at end of line
                                 # If reference is at end of line, we do not
                                 # want an extra line-feed character.
        {
                if (length($NoteBuffer) > 0)
                {        
                        if ($EndChar ne "\n")
                        {
                                $NoteBuffer = "\n" . $NoteBuffer;
                        }
                        print(OUTFILE $NoteBuffer);
                        $NoteBuffer = '';
                }
        }



#       CHECK FOR FNOTE MARKER
#          FOR EACH FNOTE
        while(/\*f([0-9]+)\*/)  # Scan the line for footnote references.
                                # There may be multiple references even
                                # at the end of the line.
	{
		$mrkval = $1;
       
		$index = $mrkval;
                s/([^\*]+\*f[0-9]+\*)//;
                $EndChar = substr ($1, -1);
                print(OUTFILE $1);
		if($ftntlst{$index} eq '')
		{
                        print(STDOUT "ERROR; No text for footnote reference $index",
                                     " in $bookid $chnum:$vsnum\n");
			$NoteBuffer = $NoteBuffer .= "\\f$mrkval ????\n";
			$errnum = ++$errnum;
		}
		else
		{
			$NoteBuffer .=$ftntlst{$index};   # idea is to append note
                                                          # for end of verse
		}
#                FOOTNOTE CONTENT BACK TO BE EMPTY
		$ftntlst{$index} = '';
	}

#   OUTPUT TEXT LINE
        if(!(/^\n$/))
	{
                $EndChar = substr ($_, -1);   # remember the last character.
                                              # It may be a line-return.
                print(OUTFILE $_);      
	}
}
#CLOSE BIBLE TEXT FILE
close(TXTFILE);
#OUTPUT THE NOTES FOR THE LAST VERSE
#   output the Footnote from NoteBuffer
if (length($NoteBuffer) > 0)
{        
        if ($EndChar ne "\n")
        {
                $NoteBuffer = "\n" . $NoteBuffer;
        }
        print(OUTFILE $NoteBuffer);
        $NoteBuffer = '';
}
close(OUTFILE); 
# Take a look at the BookName for content:
if ($bookid eq "???")
{
        print(STDOUT "ERROR; The \id marker appears to be missing?\n");
        $errnum = ++$errnum;
}
#REPORT ALL MISMATCH AND ERRORS
foreach $tmp (sort(keys(%ftntlst)))
{
# ne NOT EQUAL FOR STRINGS, != FOR NUMBERS
	if ($ftntlst{$tmp} ne '')
	{        
                print(STDOUT "ERROR; $ARGV[0]: Unreferenced footnote: [$tmp]\n",
				"    $ftntlst{$tmp}");
		$errnum = ++$errnum;
	}
}
if ($errnum eq 0)
{
        print(STDERR "$bookid : NO ERRORS\n");
        print(STDOUT "$bookid : NO ERRORS\n");

}
else
{
        print(STDERR "$bookid : ERRORS to report: $errnum \n");
        print(STDOUT "$bookid : ERRORS to report: $errnum \n");

}
